------------------------------------------------------------------------------
--  TM4C1294.GPIO                                                           --
------------------------------------------------------------------------------

with System;

package body TM4C1294.GPIO is

   type GPIOPCTL_Register is array (GPIO_Pin) of GPIO_Alt_Func;
   for GPIOPCTL_Register'Component_Size use 4;

   type GPIODATA_Register is array (Interfaces.Unsigned_8) of Word;
   for GPIODATA_Register'Component_Size use 32;

   type GPIO_Registers is record
      GPIODATA      : GPIODATA_Register;
      GPIODIR       : Word;
      GPIOIS        : Word;
      GPIOIBE       : Word;
      GPIOIEV       : Word;
      GPIOIM        : Word;
      GPIORIS       : Word;
      GPIOMIS       : Word;
      GPIOICR       : Word;
      GPIOAFSEL     : Word;
      GPIODR2R      : Word;
      GPIODR4R      : Word;
      GPIODR8R      : Word;
      GPIOODR       : Word;
      GPIOPUR       : Word;
      GPIOPDR       : Word;
      GPIOSLR       : Word;
      GPIODEN       : Word;
      GPIOLOCK      : Word;
      GPIOCR        : Word;
      GPIOAMSEL     : Word;
      GPIOPCTL      : GPIOPCTL_Register;
      GPIOADCCTL    : Word;
      GPIODMACTL    : Word;
      GPIOSI        : Word;
      GPIODR12R     : Word;
      GPIOWAKEPEN   : Word;
      GPIOWAKELVL   : Word;
      GPIOWAKESTAT  : Word;
      GPIOPP        : Word;
      GPIOPC        : Word;
      GPIOPeriphID4 : Word;
      GPIOPeriphID5 : Word;
      GPIOPeriphID6 : Word;
      GPIOPeriphID7 : Word;
      GPIOPeriphID0 : Word;
      GPIOPeriphID1 : Word;
      GPIOPeriphID2 : Word;
      GPIOPeriphID3 : Word;
      GPIOPCellID0  : Word;
      GPIOPCellID1  : Word;
      GPIOPCellID2  : Word;
      GPIOPCellID3  : Word;
   end record;

   for GPIO_Registers use record
      GPIODATA      at 16#000# range 0 .. 8191;
      GPIODIR       at 16#400# range 0 .. 31;
      GPIOIS        at 16#404# range 0 .. 31;
      GPIOIBE       at 16#408# range 0 .. 31;
      GPIOIEV       at 16#40C# range 0 .. 31;
      GPIOIM        at 16#410# range 0 .. 31;
      GPIORIS       at 16#414# range 0 .. 31;
      GPIOMIS       at 16#418# range 0 .. 31;
      GPIOICR       at 16#41C# range 0 .. 31;
      GPIOAFSEL     at 16#420# range 0 .. 31;
      GPIODR2R      at 16#500# range 0 .. 31;
      GPIODR4R      at 16#504# range 0 .. 31;
      GPIODR8R      at 16#508# range 0 .. 31;
      GPIOODR       at 16#50C# range 0 .. 31;
      GPIOPUR       at 16#510# range 0 .. 31;
      GPIOPDR       at 16#514# range 0 .. 31;
      GPIOSLR       at 16#518# range 0 .. 31;
      GPIODEN       at 16#51C# range 0 .. 31;
      GPIOLOCK      at 16#520# range 0 .. 31;
      GPIOCR        at 16#524# range 0 .. 31;
      GPIOAMSEL     at 16#528# range 0 .. 31;
      GPIOPCTL      at 16#52C# range 0 .. 31;
      GPIOADCCTL    at 16#530# range 0 .. 31;
      GPIODMACTL    at 16#534# range 0 .. 31;
      GPIOSI        at 16#538# range 0 .. 31;
      GPIODR12R     at 16#53C# range 0 .. 31;
      GPIOWAKEPEN   at 16#540# range 0 .. 31;
      GPIOWAKELVL   at 16#544# range 0 .. 31;
      GPIOWAKESTAT  at 16#548# range 0 .. 31;
      GPIOPP        at 16#FC0# range 0 .. 31;
      GPIOPC        at 16#FC4# range 0 .. 31;
      GPIOPeriphID4 at 16#FD0# range 0 .. 31;
      GPIOPeriphID5 at 16#FD4# range 0 .. 31;
      GPIOPeriphID6 at 16#FD8# range 0 .. 31;
      GPIOPeriphID7 at 16#FDC# range 0 .. 31;
      GPIOPeriphID0 at 16#FE0# range 0 .. 31;
      GPIOPeriphID1 at 16#FE4# range 0 .. 31;
      GPIOPeriphID2 at 16#FE8# range 0 .. 31;
      GPIOPeriphID3 at 16#FEC# range 0 .. 31;
      GPIOPCellID0  at 16#FF0# range 0 .. 31;
      GPIOPCellID1  at 16#FF4# range 0 .. 31;
      GPIOPCellID2  at 16#FF8# range 0 .. 31;
      GPIOPCellID3  at 16#FFC# range 0 .. 31;
   end record;

   type Port_Address_Lookup is array (GPIO_Port) of System.Address;

   Port_Addr : constant Port_Address_Lookup := (
      PortA => System'To_Address (GPIO_PORTA_Base_Address),
      PortB => System'To_Address (GPIO_PORTB_Base_Address),
      PortC => System'To_Address (GPIO_PORTC_Base_Address),
      PortD => System'To_Address (GPIO_PORTD_Base_Address),
      PortE => System'To_Address (GPIO_PORTE_Base_Address),
      PortF => System'To_Address (GPIO_PORTF_Base_Address),
      PortG => System'To_Address (GPIO_PORTG_Base_Address),
      PortH => System'To_Address (GPIO_PORTH_Base_Address),
      PortJ => System'To_Address (GPIO_PORTJ_Base_Address),
      PortK => System'To_Address (GPIO_PORTK_Base_Address),
      PortL => System'To_Address (GPIO_PORTL_Base_Address),
      PortM => System'To_Address (GPIO_PORTM_Base_Address),
      PortN => System'To_Address (GPIO_PORTN_Base_Address),
      PortP => System'To_Address (GPIO_PORTP_Base_Address),
      PortQ => System'To_Address (GPIO_PORTQ_Base_Address));

   type Pin_Masks is array (GPIO_Pin) of Word;

   Pin_Bit : constant Pin_Masks := (1, 2, 4, 8, 16, 32, 64, 128);

   use type Word;

   function Pin_Supports_Analog (Port : GPIO_Port;
                                 Pin  : GPIO_Pin) return Boolean is
      Result : Boolean;
   begin

      case Port is

         when PortB => -- AIN 10,11
            Result := (Pin in 4 .. 5);

         when PortD => -- AIN 4,5,6,7,12,13,14,15
            Result := True;

         when PortE => -- AIN 0,1,2,3,8,9
            Result := (Pin <= 5);

         when PortK => -- AIN 16,17,18,19
            Result := (Pin <= 3);

         when others =>
            Result := False;

      end case;

      return Result;
   end Pin_Supports_Analog;

-----------------------------------------------------------------------------
--  GPIO_Write                                                             --
-----------------------------------------------------------------------------

   procedure GPIO_Write (Port        : GPIO_Port;
                         Value       : Interfaces.Unsigned_8;
                         Output_Mask : Interfaces.Unsigned_8 := 16#FF#) is
      Regs : GPIO_Registers with Volatile, Address => Port_Addr (Port);
   begin
      Regs.GPIODATA (Output_Mask) := Word (Value);
   end GPIO_Write;

------------------------------------------------------------------------------
--  GPIO_Set_Mode                                                           --
------------------------------------------------------------------------------

   procedure GPIO_Set_Mode (Port : GPIO_Port;
                            Pin  : GPIO_Pin;
                            Mode : GPIO_Mode;
                            AF   : GPIO_Alt_Func := 0) is
      Regs : GPIO_Registers with Volatile, Address => Port_Addr (Port);
   begin

--      Regs.GPIOLOCK := 16#4C4F_434B#;

--      loop
--         exit when Regs.GPIOLOCK = 0;
--      end loop;

--      Regs.GPIOCR := Regs.GPIOCR or 1;

      case Mode is

         when Off =>

            Regs.GPIODEN := Regs.GPIODEN and not Pin_Bit (Pin);
            Regs.GPIOAMSEL := Regs.GPIOAMSEL and not Pin_Bit (Pin);
            Regs.GPIOAFSEL := Regs.GPIOAFSEL and not Pin_Bit (Pin);

         when Input =>
            Regs.GPIOAMSEL := Regs.GPIOAMSEL and not Pin_Bit (Pin);
            Regs.GPIOAFSEL := Regs.GPIOAFSEL and not Pin_Bit (Pin);
            Regs.GPIODIR := Regs.GPIODIR and not Pin_Bit (Pin);
            Regs.GPIODEN := Regs.GPIODEN or Pin_Bit (Pin);

         when Output =>
            Regs.GPIOAMSEL := Regs.GPIOAMSEL and not Pin_Bit (Pin);
            Regs.GPIOAFSEL := Regs.GPIOAFSEL and not Pin_Bit (Pin);
            Regs.GPIODIR := Regs.GPIODIR or Pin_Bit (Pin);
            Regs.GPIODEN := Regs.GPIODEN or Pin_Bit (Pin);

         when Alt_Function =>
            Regs.GPIOAMSEL := Regs.GPIOAMSEL and not Pin_Bit (Pin);
            Regs.GPIOPCTL (Pin) := AF;
            Regs.GPIOAFSEL := Regs.GPIOAFSEL or Pin_Bit (Pin);
            Regs.GPIODEN := Regs.GPIODEN or Pin_Bit (Pin);

         when Analog =>
            if Pin_Supports_Analog  (Port, Pin) then
               Regs.GPIODEN := Regs.GPIODEN and not Pin_Bit (Pin);
               Regs.GPIOAFSEL := Regs.GPIOAFSEL and not Pin_Bit (Pin);
               Regs.GPIOAMSEL := Regs.GPIOAMSEL or Pin_Bit (Pin);
            end if;

      end case;

--      Regs.GPIOCR := Regs.GPIOCR and 16#FFFF_FFFE#;
--      Regs.GPIOLOCK := 16#0000_0000#;

   end GPIO_Set_Mode;

   procedure GPIO_Set_Drive (Port  : GPIO_Port;
                             Pin   : GPIO_Pin;
                             Drive : GPIO_Drive) is
      Regs : GPIO_Registers with Volatile, Address => Port_Addr (Port);
   begin
      case Drive is

         when Drive_2mA =>
            Regs.GPIODR2R := Regs.GPIODR2R or Pin_Bit (Pin);

         when Drive_4mA =>
            Regs.GPIODR4R := Regs.GPIODR4R or Pin_Bit (Pin);

         when Drive_8mA =>
            Regs.GPIODR8R := Regs.GPIODR8R or Pin_Bit (Pin);

      end case;
   end GPIO_Set_Drive;

------------------------------------------------------------------------------
--  Wait                                                                    --
------------------------------------------------------------------------------

   procedure Wait is
      Counter : Natural := 0;
      pragma Volatile (Counter);
   begin
      loop
         Counter := Counter + 1;
         exit when Counter > 2_000_000;
      end loop;
   end Wait;

------------------------------------------------------------------------------
--  Blink_LED1                                                              --
------------------------------------------------------------------------------

   procedure Blink_LED1 is
   begin

      Enable_Port_Clocking (PortN);

      GPIO_Set_Mode (PortN, 0, Output);

      GPIO_Write (PortN, 1, 1);

      Wait;

      GPIO_Write (PortN, 0, 1);

      Wait;

   end Blink_LED1;

------------------------------------------------------------------------------
--  Blink_LED2                                                              --
------------------------------------------------------------------------------

   procedure Blink_LED2 is
   begin

      Enable_Port_Clocking (PortN);

      GPIO_Set_Mode (PortN, 1, Output);

      GPIO_Write (PortN, 2, 2);

      Wait;

      GPIO_Write (PortN, 0, 2);

      Wait;

   end Blink_LED2;

end TM4C1294.GPIO;
