-----------------------------------------------------------------------------
--  Ti Tiva TM4C1294 UART driver                                           --
-----------------------------------------------------------------------------

with System.Storage_Elements;
with System.Address_To_Access_Conversions;

package TM4C1294.UART is
   pragma Preelaborate;

   package SSE renames System.Storage_Elements;

   subtype UART_Data_Bits is Positive range 5 .. 8;

   type UART_Stop_Bits is (One, Two);

   type UART_Parity is (Odd, Even);

   type UART_IO (Base_Addr : SSE.Integer_Address) is private;

   function UART_Tx_Ready (UART : UART_IO) return Boolean;

   function UART_Rx_Ready (UART : UART_IO) return Boolean;

   function UART_Get (UART : UART_IO) return Character;

   procedure UART_Put (UART : in out UART_IO; C : Character);

   procedure UART_Put (UART : in out UART_IO; S : String);

   procedure UART_Enable (UART : in out UART_IO);

   procedure UART_Disable (UART : in out UART_IO);

   procedure UART_Enable_FIFO (UART : in out UART_IO);

   procedure UART_Set_Baud_Rate (UART        : in out UART_IO;
                                 Baud_Rate   : Positive;
                                 Use_Sys_Clk : Boolean := False;
                                 Clk_Freq    : Positive := 16_000_000);

   procedure UART_Configure (UART      : in out UART_IO;
                             Data_Bits : UART_Data_Bits := 8;
                             Stop_Bits : UART_Stop_Bits := One;
                             Parity_On : Boolean := False;
                             Parity    : UART_Parity := Even);

private

   type UART_Registers is record
      UARTDR          : Word;
      UARTRSR_UARTECR : Word;
      UARTFR          : Word;
      UARTILPR        : Word;
      UARTIBRD        : Word;
      UARTFBRD        : Word;
      UARTLCRH        : Word;
      UARTCTL         : Word;
      UARTIFLS        : Word;
      UARTIM          : Word;
      UARTRIS         : Word;
      UARTMIS         : Word;
      UARTICR         : Word;
      UARTDMACTL      : Word;
      UART9BITADDR    : Word;
      UART9BITAMASK   : Word;
      UARTPP          : Word;
      UARTCC          : Word;
      UARTPeriphID4   : Word;
      UARTPeriphID5   : Word;
      UARTPeriphID6   : Word;
      UARTPeriphID7   : Word;
      UARTPeriphID0   : Word;
      UARTPeriphID1   : Word;
      UARTPeriphID2   : Word;
      UARTPeriphID3   : Word;
      UARTPCellID0    : Word;
      UARTPCellID1    : Word;
      UARTPCellID2    : Word;
      UARTPCellID3    : Word;
   end record;

   for UART_Registers use record
      UARTDR          at 16#000# range 0 .. 31;
      UARTRSR_UARTECR at 16#004# range 0 .. 31;
      UARTFR          at 16#018# range 0 .. 31;
      UARTILPR        at 16#020# range 0 .. 31;
      UARTIBRD        at 16#024# range 0 .. 31;
      UARTFBRD        at 16#028# range 0 .. 31;
      UARTLCRH        at 16#02C# range 0 .. 31;
      UARTCTL         at 16#030# range 0 .. 31;
      UARTIFLS        at 16#034# range 0 .. 31;
      UARTIM          at 16#038# range 0 .. 31;
      UARTRIS         at 16#03C# range 0 .. 31;
      UARTMIS         at 16#040# range 0 .. 31;
      UARTICR         at 16#044# range 0 .. 31;
      UARTDMACTL      at 16#048# range 0 .. 31;
      UART9BITADDR    at 16#0A4# range 0 .. 31;
      UART9BITAMASK   at 16#0A8# range 0 .. 31;
      UARTPP          at 16#FC0# range 0 .. 31;
      UARTCC          at 16#FC8# range 0 .. 31;
      UARTPeriphID4   at 16#FD0# range 0 .. 31;
      UARTPeriphID5   at 16#FD4# range 0 .. 31;
      UARTPeriphID6   at 16#FD8# range 0 .. 31;
      UARTPeriphID7   at 16#FDC# range 0 .. 31;
      UARTPeriphID0   at 16#FE0# range 0 .. 31;
      UARTPeriphID1   at 16#FE4# range 0 .. 31;
      UARTPeriphID2   at 16#FE8# range 0 .. 31;
      UARTPeriphID3   at 16#FEC# range 0 .. 31;
      UARTPCellID0    at 16#FF0# range 0 .. 31;
      UARTPCellID1    at 16#FF4# range 0 .. 31;
      UARTPCellID2    at 16#FF8# range 0 .. 31;
      UARTPCellID3    at 16#FFC# range 0 .. 31;
   end record;

   pragma Volatile (UART_Registers);

   package RegPtr is new
      System.Address_To_Access_Conversions (UART_Registers);

   type UART_IO (Base_Addr : SSE.Integer_Address) is record
      Regs : RegPtr.Object_Pointer :=
                RegPtr.To_Pointer (SSE.To_Address (Base_Addr));
   end record;

end TM4C1294.UART;
