------------------------------------------------------------------------------
--                                                                          --
--                  GNAT RUN-TIME LIBRARY (GNARL) COMPONENTS                --
--                                                                          --
--                   A D A . I N T E R R U P T S . N A M E S                --
--                                                                          --
--                                  S p e c                                 --
--                                                                          --
--          Copyright (C) 1991-2013, Free Software Foundation, Inc.         --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 3,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.                                     --
--                                                                          --
--                                                                          --
--                                                                          --
--                                                                          --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
-- GNARL was developed by the GNARL team at Florida State University.       --
-- Extensive contributions were provided by Ada Core Technologies, Inc.     --
--                                                                          --
------------------------------------------------------------------------------

--  This is the version for Cortex M4F STM32F4 targets

package Ada.Interrupts.Names is

   --  All identifiers in this unit are implementation defined

   pragma Implementation_Defined;

   --  The STM32F4X reference manual defines the interrupt in Table 45
   --  (page 250 of Doc ID 018909 Rev 4). The meaningful number, the position
   --  starts at 0. Unfortunately, Interrupt_ID 0 is reserved and the SysTick
   --  interrupt (a core interrupt) is handled by the runtime like other
   --  interrupts. So the first interrupt (window watchdog) is numbered 2 while
   --  it is at position 0 in the manual. The offset of 2 is reflected in
   --  s-bbbosu-stm32f4.adb by the First_IRQ constant.

   Sys_Tick_Interrupt               : constant Interrupt_ID := 1;
   GPIO_Port_A_Interrupt            : constant Interrupt_ID := 2;
   GPIO_Port_B_Interrup             : constant Interrupt_ID := 3;
   GPIO_Port_C_Interrup             : constant Interrupt_ID := 4;
   GPIO_Port_D_Interrup             : constant Interrupt_ID := 5;
   GPIO_Port_E_Interrup             : constant Interrupt_ID := 6;
   UART_0_Interrupt                 : constant Interrupt_ID := 7;
   UART_1_Interrupt                 : constant Interrupt_ID := 8;
   SSI_0_Interrupt                  : constant Interrupt_ID := 9;
   I2C_0_Interrupt                  : constant Interrupt_ID := 10;
   PWM_Fault_Interrupt              : constant Interrupt_ID := 11;
   PWM_GEN_0_Interrupt              : constant Interrupt_ID := 12;
   PWM_GEN_1_Interrupt              : constant Interrupt_ID := 13;
   PWM_GEN_2_Interrupt              : constant Interrupt_ID := 14;
   QEI_0_Interrupt                  : constant Interrupt_ID := 15;
   ADC_0_Sequence_0_Interrupt       : constant Interrupt_ID := 16;
   ADC_0_Sequence_1_Interrupt       : constant Interrupt_ID := 17;
   ADC_0_Sequence_2_Interrupt       : constant Interrupt_ID := 18;
   ADC_0_Sequence_3_Interrupt       : constant Interrupt_ID := 19;
   Watchdog_Timer_Interrupt         : constant Interrupt_ID := 20;
   Timer_0A_Interrupt               : constant Interrupt_ID := 21;
   Timer_0B_Interrupt               : constant Interrupt_ID := 22;
   Timer_1A_Interrupt               : constant Interrupt_ID := 23;
   Timer_1B_Interrupt               : constant Interrupt_ID := 24;
   Timer_2A_Interrupt               : constant Interrupt_ID := 25;
   Timer_2B_Interrupt               : constant Interrupt_ID := 26;
   Analog_Comparator_0_Interrupt    : constant Interrupt_ID := 27;
   Analog_Comparator_1_Interrupt    : constant Interrupt_ID := 28;
   Analog_Comparator_2_Interrupt    : constant Interrupt_ID := 29;
   System_Control_Interrupt         : constant Interrupt_ID := 30;
   Flash_Memory_Control_Interrupt   : constant Interrupt_ID := 31;
   GPIO_Port_F_Interrupt            : constant Interrupt_ID := 32;
   GPIO_Port_G_Interrupt            : constant Interrupt_ID := 33;
   GPIO_Port_H_Interrupt            : constant Interrupt_ID := 34;
   UART_2_Interrupt                 : constant Interrupt_ID := 35;
   SSI_1_Interrupt                  : constant Interrupt_ID := 36;
   Timer_3A_Interrupt               : constant Interrupt_ID := 37;
   Timer_3B_Interrupt               : constant Interrupt_ID := 38;
   I2C_1_Interrupt                  : constant Interrupt_ID := 39;
   CAN_0_Interrupt                  : constant Interrupt_ID := 40;
   CAN_1_Interrupt                  : constant Interrupt_ID := 41;
   Ethernet_MAC_Interrupt           : constant Interrupt_ID := 42;
   HIB_Interrupt                    : constant Interrupt_ID := 43;
   USB_MAC_Interrupt                : constant Interrupt_ID := 44;
   PWM_GEN_3_Interrupt              : constant Interrupt_ID := 45;
   DMA_0_Software_Interrupt         : constant Interrupt_ID := 46;
   DMA_0_Error_Interrupt            : constant Interrupt_ID := 47;
   ADC_1_Sequence_0_Interrupt       : constant Interrupt_ID := 48;
   ADC_1_Sequence_1_Interrupt       : constant Interrupt_ID := 49;
   ADC_1_Sequence_2_Interrupt       : constant Interrupt_ID := 50;
   ADC_1_Sequence_3_Interrupt       : constant Interrupt_ID := 51;
   EPI_0_Interrupt                  : constant Interrupt_ID := 52;
   GPIO_Port_J_Interrupt            : constant Interrupt_ID := 53;
   GPIO_Port_K_Interrupt            : constant Interrupt_ID := 54;
   GPIO_Port_L_Interrupt            : constant Interrupt_ID := 55;
   SSI_2_Interrupt                  : constant Interrupt_ID := 56;
   SSI_3_Interrupt                  : constant Interrupt_ID := 57;
   UART_3_Interrupt                 : constant Interrupt_ID := 58;
   UART_4_Interrupt                 : constant Interrupt_ID := 59;
   UART_5_Interrupt                 : constant Interrupt_ID := 60;
   UART_6_Interrupt                 : constant Interrupt_ID := 61;
   UART_7_Interrupt                 : constant Interrupt_ID := 62;
   I2C_2_Interrupt                  : constant Interrupt_ID := 63;
   I2C_3_Interrupt                  : constant Interrupt_ID := 64;
   Timer_4A_Interrupt               : constant Interrupt_ID := 65;
   Timer_4B_Interrupt               : constant Interrupt_ID := 66;
   Timer_5A_Interrupt               : constant Interrupt_ID := 67;
   Timer_5B_Interrupt               : constant Interrupt_ID := 68;
   FP_Exception_Interrupt           : constant Interrupt_ID := 69;
   Reserved_1_Interrupt             : constant Interrupt_ID := 70;
   Reserved_2_Interrupt             : constant Interrupt_ID := 71;
   I2C_4_Interrupt                  : constant Interrupt_ID := 72;
   I2C_5_Interrupt                  : constant Interrupt_ID := 73;
   GPIO_Port_M_Interrupt            : constant Interrupt_ID := 74;
   GPIO_Port_N_Interrupt            : constant Interrupt_ID := 75;
   Reserved_3_Interrupt             : constant Interrupt_ID := 76;
   Tamper_Interrupt                 : constant Interrupt_ID := 77;
   GPIO_Port_P0_Interrupt           : constant Interrupt_ID := 78;
   GPIO_Port_P1_Interrupt           : constant Interrupt_ID := 79;
   GPIO_Port_P2_Interrupt           : constant Interrupt_ID := 80;
   GPIO_Port_P3_Interrupt           : constant Interrupt_ID := 81;
   GPIO_Port_P4_Interrupt           : constant Interrupt_ID := 82;
   GPIO_Port_P5_Interrupt           : constant Interrupt_ID := 83;
   GPIO_Port_P6_Interrupt           : constant Interrupt_ID := 84;
   GPIO_Port_P7_Interrupt           : constant Interrupt_ID := 85;
   GPIO_Port_Q0_Interrupt           : constant Interrupt_ID := 86;
   GPIO_Port_Q1_Interrupt           : constant Interrupt_ID := 87;
   GPIO_Port_Q2_Interrupt           : constant Interrupt_ID := 88;
   GPIO_Port_Q3_Interrupt           : constant Interrupt_ID := 89;
   GPIO_Port_Q4_Interrupt           : constant Interrupt_ID := 90;
   GPIO_Port_Q5_Interrupt           : constant Interrupt_ID := 91;
   GPIO_Port_Q6_Interrupt           : constant Interrupt_ID := 92;
   GPIO_Port_Q7_Interrupt           : constant Interrupt_ID := 93;
   Reserved_4_Interrupt             : constant Interrupt_ID := 94;
   Reserved_5_Interrupt             : constant Interrupt_ID := 95;
   Reserved_6_Interrupt             : constant Interrupt_ID := 96;
   Reserved_7_Interrupt             : constant Interrupt_ID := 97;
   Reserved_8_Interrupt             : constant Interrupt_ID := 98;
   Reserved_9_Interrupt             : constant Interrupt_ID := 99;
   Timer_6A_Interrupt               : constant Interrupt_ID := 100;
   Timer_6B_Interrupt               : constant Interrupt_ID := 101;
   Timer_7A_Interrupt               : constant Interrupt_ID := 102;
   Timer_7B_Interrupt               : constant Interrupt_ID := 103;
   I2C_6_Interrupt                  : constant Interrupt_ID := 104;
   I2C_7_Interrupt                  : constant Interrupt_ID := 105;
   Reserved_10_Interrupt            : constant Interrupt_ID := 106;
   Reserved_11_Interrupt            : constant Interrupt_ID := 107;
   Reserved_12_Interrupt            : constant Interrupt_ID := 108;
   Reserved_13_Interrupt            : constant Interrupt_ID := 109;
   Reserved_14_Interrupt            : constant Interrupt_ID := 110;
   I2C_8_Interrupt                  : constant Interrupt_ID := 111;
   I2C_9_Interrupt                  : constant Interrupt_ID := 112;
   Reserved_15_Interrupt            : constant Interrupt_ID := 113;
   Reserved_16_Interrupt            : constant Interrupt_ID := 114;
   Reserved_17_Interrupt            : constant Interrupt_ID := 115;

end Ada.Interrupts.Names;
